@@ -5,7 +5,7 @@ use strict;
 use Carp qw(cluck);
 use Bio::ToolBox::Data;
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 sub load_file {
 	my $self = shift;
@@ -47,12 +47,6 @@ sub new_data {
 	}
 }
 
-sub open_file {
-	my $self = shift;
-	cluck('Bio::ToolBox->open_file has been replaced by read_file');
-	return Bio::ToolBox::Data->open_to_read_fh(@_);
-}
-
 sub read_file {
 	my $self = shift;
 	return Bio::ToolBox::Data->open_to_read_fh(@_);
@@ -68,16 +62,6 @@ sub open_database {
 	return Bio::ToolBox::Data->open_new_database(@_);
 }
 
-sub bam_adapter {
-	my $self = shift;
-	return Bio::ToolBox::Data->bam_adapter(@_);
-}
-
-sub big_adapter {
-	my $self = shift;
-	return Bio::ToolBox::Data->big_adapter(@_);
-}
-
 1;
 
 __END__
