@@ -2,87 +2,75 @@
 
 # documentation at end of file
 
+use warnings;
 use strict;
 use Getopt::Long qw(:config no_ignore_case bundling);
 use Pod::Usage;
+use IO::Prompt::Tiny qw(prompt);
 use Bio::ToolBox::Data;
-use Bio::ToolBox::utility;
-my $VERSION =  '1.62';
+use Bio::ToolBox::utility qw(parse_list ask_user_for_index format_with_commas);
+
+our $VERSION = '2.00';
 
 print "\n This script will convert a data file to a GFF\n\n";
 
-
 ### Quick help
-unless (@ARGV) { 
+unless (@ARGV) {
+
 	# when no command line options are present
 	# print SYNOPSIS
-	pod2usage( {
-		'-verbose' => 0, 
-		'-exitval' => 1,
-	} );
+	pod2usage(
+		{
+			'-verbose' => 0,
+			'-exitval' => 1,
+		}
+	);
 }
 
-
-
 ### Get command line options and initialize values
 my (
-	$infile, 
-	$outfile,
-	$no_header,
-	$chr_index,
-	$start_index,
-	$stop_index,
-	$score_index,
-	$strand_index,
-	$name_index,
-	$name,
-	$id_index,
-	$source,
-	$type,
-	$tag,
-	$ask,
-	$unique,
-	$interbase,
-	$sort_data,
-	$gz,
-	$bgz,
-	$help,
+	$infile,     $outfile,     $no_header,    $chr_index, $start_index,
+	$stop_index, $score_index, $strand_index, $use_name,  $id_index,
+	$source,     $type,        $tag,          $ask,       $unique,
+	$interbase,  $sort_data,   $gz,           $bgz,       $help,
 	$print_version,
 );
 
-
 # Command line options
-GetOptions( 
-	'i|in=s'      => \$infile, # specify the input data file
-	'o|out=s'     => \$outfile, # name of output gff file 
-	'H|noheader'  => \$no_header, # source has no header line
-	'c|chr=i'     => \$chr_index, # index of the chromosome column
-	'b|begin|start=i'   => \$start_index, # index of the start position column
-	'e|stop|end=i'=> \$stop_index, # index of the stop position coloumn
-	's|score=i'   => \$score_index, # index for the score column
-	't|strand=i'  => \$strand_index, # index for the strand column
-	'n|name=s'    => \$name, # index for the name column or the name text
-	'd|id=i'      => \$id_index, # index for the ID column
-	'r|source=s'  => \$source, # text to put in the source column
-	'y|type=s'    => \$type, # test to put in the type column
-	'g|tag|tags=s'=> \$tag, # comma list of tag column indices
-	'a|ask'       => \$ask, # request help in assigning indices
-	'unique!'     => \$unique, # make the names unique
-	'0|zero!'     => \$interbase, # input file is interbase format
-	'sort!'       => \$sort_data, # sort the output file
-	'z|gz!'       => \$gz, # boolean to compress output file
-	'Z|bgz!'      => \$bgz, # compress with bgzip
-	'h|help'      => \$help, # request help
-	'v|version'   => \$print_version, # print the version
+GetOptions(
+	'i|in=s'          => \$infile,           # specify the input data file
+	'o|out=s'         => \$outfile,          # name of output gff file
+	'H|noheader'      => \$no_header,        # source has no header line
+	'c|chr=i'         => \$chr_index,        # index of the chromosome column
+	'b|begin|start=i' => \$start_index,      # index of the start position column
+	'e|stop|end=i'    => \$stop_index,       # index of the stop position coloumn
+	's|score=i'       => \$score_index,      # index for the score column
+	't|strand=i'      => \$strand_index,     # index for the strand column
+	'n|name=s'        => \$use_name,         # index for the name column or the name text
+	'd|id=i'          => \$id_index,         # index for the ID column
+	'r|source=s'      => \$source,           # text to put in the source column
+	'y|type=s'        => \$type,             # test to put in the type column
+	'g|tag|tags=s'    => \$tag,              # comma list of tag column indices
+	'a|ask'           => \$ask,              # request help in assigning indices
+	'unique!'         => \$unique,           # make the names unique
+	'0|zero!'         => \$interbase,        # input file is interbase format
+	'sort!'           => \$sort_data,        # sort the output file
+	'z|gz!'           => \$gz,               # boolean to compress output file
+	'Z|bgz!'          => \$bgz,              # compress with bgzip
+	'h|help'          => \$help,             # request help
+	'v|version'       => \$print_version,    # print the version
 ) or die " unrecognized option(s)!! please refer to the help documentation\n\n";
 
 # Print help
 if ($help) {
+
 	# print entire POD
-	pod2usage( {
-		'-verbose' => 2,
-		'-exitval' => 1,
-	} );
+	pod2usage(
+		{
+			'-verbose' => 2,
+			'-exitval' => 1,
+		}
+	);
 }
 
 # Print version
@@ -96,52 +84,61 @@ if ($print_version) {
 	exit;
 }
 
-
-
 ### Check for required values
 unless ($infile) {
-	$infile = shift @ARGV or
-		die "  OOPS! No source data file specified! \n use $0 --help\n";
+	if (@ARGV) {
+		$infile = shift @ARGV;
+	}
+	else {
+		print STDERR " FATAL: no input file! use --help for more information\n";
+		exit 1;
+	}
 }
 if ($bgz) {
-	$gz = 2;
+	$gz        = 2;
 	$sort_data = 1;
 }
 
 # define name base or index
-my ($name_index, $name_base);
-if (defined $name) {
-	if ($name =~ /^(\d+)$/) {
+my ( $name_index, $name_base );
+if ( defined $use_name ) {
+	if ( $use_name =~ /^(\d+)$/ ) {
+
 		# looks like an index was provided
 		$name_index = $1;
 	}
-	elsif ($name =~ /(\w+)/i) {
+	elsif ( $use_name =~ /(\w+)/i ) {
+
 		# text that will be used as the name base when autogenerating
 		$name_base = $1;
 	}
 }
 
 # define type base or index
-my ($type_index, $type_base);
-if (defined $type) {
-	if ($type =~ /^(\d+)$/) {
+my ( $type_index, $type_base );
+if ( defined $type ) {
+	if ( $type =~ /^(\d+)$/ ) {
+
 		# looks like an index was provided
 		$type_index = $1;
 	}
-	elsif ($type =~ /(\w+)/i) {
+	elsif ( $type =~ /(\w+)/i ) {
+
 		# text that will be used as the type base when autogenerating
 		$type_base = $1;
 	}
 }
 
 # define source base or index
-my ($source_index, $source_base);
-if (defined $source) {
-	if ($source =~ /^(\d+)$/) {
+my ( $source_index, $source_base );
+if ( defined $source ) {
+	if ( $source =~ /^(\d+)$/ ) {
+
 		# looks like an index was provided
 		$source_index = $1;
 	}
-	elsif ($source =~ /(\w+)/i) {
+	elsif ( $source =~ /(\w+)/i ) {
+
 		# text that will be used as the source base when autogenerating
 		$source_base = $1;
 	}
@@ -153,231 +150,237 @@ if ($tag) {
 	@tag_indices = parse_list($tag);
 }
 
-
-
-
 ### Load file
 my $Input = Bio::ToolBox::Data->new(
-	in       => $infile, 
+	in       => $infile,
 	noheader => $no_header,
 	stream   => 1,
 ) or die "Unable to open file '$infile'!\n";
 
 ### Determine indices
 if ($ask) {
+
 	# the user has specified that we should ask for specific indices
 	print " Press Return to accept the suggested index\n";
-	
+
 	# request chromosome index
-	unless (defined $chr_index) {
+	unless ( defined $chr_index ) {
 		my $suggestion = $Input->chromo_column;
-		$chr_index = ask_user_for_index($Input, 
-			" Enter the index for the chromosome column [$suggestion]  ");
+		$chr_index = ask_user_for_index( $Input,
+			" Enter the index for the chromosome column [$suggestion]: " );
 		$chr_index = defined $chr_index ? $chr_index : $suggestion;
-		unless (defined $chr_index) {
-			die " No identifiable chromosome column index!\n";
+		unless ( defined $chr_index ) {
+			print STDERR " FATAL: No identifiable chromosome column index!\n";
+			exit 1;
 		}
 	}
-	
+
 	# request start index
-	unless (defined $start_index) {
+	unless ( defined $start_index ) {
 		my $suggestion = $Input->start_column;
-		$start_index = ask_user_for_index($Input, 
-			" Enter the index for the start column [$suggestion]  ");
+		$start_index = ask_user_for_index( $Input,
+			" Enter the index for the start column [$suggestion]: " );
 		$start_index = defined $start_index ? $start_index : $suggestion;
-		unless (defined $start_index) {
-			die " No identifiable start position column index!\n";
+		unless ( defined $start_index ) {
+			print STDERR " FATAL: No identifiable start position column index!\n";
+			exit 1;
 		}
 	}
-	
+
 	# request stop index
-	unless (defined $stop_index) {
+	unless ( defined $stop_index ) {
 		my $suggestion = $Input->stop_column;
-		$stop_index = ask_user_for_index($Input, 
-			" Enter the index for the stop or end column [$suggestion]  ");
+		$stop_index = ask_user_for_index( $Input,
+			" Enter the index for the stop or end column [$suggestion]: " );
 		$stop_index = defined $stop_index ? $stop_index : $suggestion;
-		unless (defined $stop_index) {
-			die " No identifiable stop position column index!\n";
+		unless ( defined $stop_index ) {
+			print STDERR " FATAL: No identifiable stop position column index!\n";
+			exit 1;
 		}
 	}
-	
+
 	# request source text
-	unless (defined $source) {
+	unless ( defined $source ) {
+
 		# this is a special input, can't use the ask_user_for_index sub
 		# accepts either index or text string
-		print " Enter the text string or column index for the GFF source (Suggested)  ";
-		my $in = <STDIN>;
-		if ($in =~ /^(\d+)$/) {
+		my $default = $Input->basename;
+		my $p  = " Enter the text string or column index for the GFF source [$default]: ";
+		my $in = prompt( $p, $default );
+		if ( $in =~ /^(\d+)$/ ) {
 			$source_index = $1;
 		}
-		elsif ($in =~ /(\w+)/) {
-			$source_base = $1;
+		else {
+			$source_base = $in;
 		}
 	}
-	
+
 	# request type text
-	unless (defined $type) {
+	unless ( defined $type ) {
+
 		# this is a special input, can't use the ask_user_for_index sub
 		# accepts either index or text string
-		print " Enter the text string or column index for the GFF type (Suggested)  ";
-		my $in = <STDIN>;
-		if ($in =~ /^(\d+)$/) {
+		my $default = $Input->feature || 'feature';
+		my $p  = " Enter the text string or column index for the GFF type [$default]: ";
+		my $in = prompt( $p, $default );
+		if ( $in =~ /^(\d+)$/ ) {
 			$type_index = $1;
 		}
-		elsif ($in =~ /(\w+)/) {
-			$type_base = $1;
+		else {
+			$type_base = $in;
 		}
 	}
-	
+
 	# request score index
-	unless (defined $score_index) {
+	unless ( defined $score_index ) {
 		my $suggestion = $Input->find_column('^score$');
-		$score_index = ask_user_for_index($Input, 
-			" Enter the index for the feature score column [$suggestion]  ");
+		$score_index = ask_user_for_index( $Input,
+			" Enter the index for the feature score column [$suggestion]: " );
 		$score_index = defined $score_index ? $score_index : $suggestion;
 	}
-	
+
 	# request strand index
-	unless (defined $strand_index) {
+	unless ( defined $strand_index ) {
 		my $suggestion = $Input->strand_column;
-		$strand_index = ask_user_for_index($Input, 
-			" Enter the index for the feature strand column [$suggestion]  ");
+		$strand_index = ask_user_for_index( $Input,
+			" Enter the index for the feature strand column [$suggestion]: " );
 		$strand_index = defined $strand_index ? $strand_index : $suggestion;
 	}
-	
+
 	# request name index or text
-	unless (defined $name) {
+	unless ( defined $use_name ) {
+
 		# this is a special input, can't use the ask_user_for_index sub
 		# accepts either index or text string
 		my $suggestion = $Input->name_column;
-		print " Enter the index for the feature name column or\n" . 
-			"   the base text for auto-generated names [$suggestion]  ";
-		my $in = <STDIN>;
-		if ($in =~ /^(\d+)$/) {
+		my $p          = " Enter the index for the feature name column or\n"
+			. "   the base text for auto-generated names: ";
+		my $in = prompt( $p, $suggestion );
+		if ( $in =~ /^(\d+)$/ ) {
 			$name_index = $1;
 		}
-		elsif ($in =~ /(\w+)/) {
-			$name_base = $1;
-		}
-		elsif (defined $suggestion) {
-			$name_index = $suggestion;
+		elsif ( $in =~ /\w+/ ) {
+			$name_base = $in;
 		}
 	}
-	
+
 	# request ID index
-	unless (defined $id_index) {
+	unless ( defined $id_index ) {
 		my $suggestion = $Input->id_column;
-		$id_index = ask_user_for_index($Input, 
-			" Enter the index for the feature unique ID column [$suggestion]  ");
+		$suggestion = q() unless $suggestion;
+		$id_index   = ask_user_for_index( $Input,
+			" Enter the index for the feature unique ID column [$suggestion]: " );
 		$id_index = defined $id_index ? $id_index : $suggestion;
 	}
-	
+
 	# request tags
-	unless (defined $tag) {
-		@tag_indices = ask_user_for_index($Input, 
-			" Enter zero or more column indices for GFF group tags  ");
+	unless ( defined $tag ) {
+		@tag_indices = ask_user_for_index( $Input,
+			" Enter zero or more column indices for GFF group tags  " );
 	}
 }
 else {
 	# or else the indices need to be automatically identified
-	unless (
-		$Input->feature_type eq 'coordinate' or 
-		(defined $chr_index and defined $start_index and defined $stop_index) or 
-		($Input->feature_type eq 'named' and $Input->database) 
-	) {
-		die "Not enough information has been provided to convert to GFF file.\n" . 
-			"Coordinate column names must be recognizable or specified. Use --help\n";
+	unless ( $Input->feature_type eq 'coordinate'
+		or ( defined $chr_index and defined $start_index and defined $stop_index )
+		or ( $Input->feature_type eq 'named' and $Input->database ) )
+	{
+		print STDERR
+			" FATAL: Not enough information has been provided to convert to GFF file.\n"
+			. "Coordinate column names must be recognizable or specified. Use --help\n";
+		exit;
 	}
 }
 
-
-
 ### Open output data
-my $Output = Bio::ToolBox::Data->new(
-	gff     => 3,
-) or die " unable to create output data strucutre!\n";
-
-
+my $Output = Bio::ToolBox::Data->new( gff => 3, )
+	or die " unable to create output data strucutre!\n";
 
 ### Convert the input stream
 # check some things first
-my $do_feature = $Input->feature_type eq 'named' ? 1 : 0; # get features from db?
-if (defined $start_index and substr($Input->name($start_index), -1) eq '0') {
+my $do_feature = $Input->feature_type eq 'named' ? 1 : 0;    # get features from db?
+if ( defined $start_index and substr( $Input->name($start_index), -1 ) eq '0' ) {
+
 	# start column name suggests it is 0-based
 	$interbase = 1;
 }
-if ($unique and not (defined $name_index or $name_base)) {
-	die " must provide a name index or name base to make unique feature names!\n";
+if ( $unique and not( defined $name_index or $name_base ) ) {
+	print STDERR
+		" FATAL: must provide a name index or name base to make unique feature names!\n";
+	exit 1;
 }
-my $unique_name_counter = {}; # hash for making unique feature names
-my $unique_id_counter   = {}; # same for IDs
-my $count = 0; # the number of lines processed
-while (my $row = $Input->next_row) {
-	
+my $unique_name_counter = {};    # hash for making unique feature names
+my $unique_id_counter   = {};    # same for IDs
+my $count               = 0;     # the number of lines processed
+while ( my $row = $Input->next_row ) {
+
 	# get the feature from the db if necessary
-	my $f = $row->feature if $do_feature;
-	
+	if ($do_feature) {
+		my $f = $row->feature;
+	}
+
 	# build the arguments
 	# retrieve information from row object if indices were provided
 	my @args;
-	if (defined $chr_index) {
+	if ($chr_index) {
 		my $c = $row->value($chr_index);
 		next if $c eq '.';
-		push @args, 'chromo', $c; 
+		push @args, 'chromo', $c;
 	}
-	if (defined $start_index) {
+	if ($start_index) {
 		my $s = $row->value($start_index);
-		next if $s eq '.';
+		next    if $s eq '.';
 		$s += 1 if $interbase;
 		push @args, 'start', $s;
 	}
-	if (defined $stop_index) {
+	if ($stop_index) {
 		push @args, 'stop', $row->value($stop_index);
 	}
-	if (defined $strand_index) {
+	if ($strand_index) {
 		push @args, 'strand', $row->value($strand_index);
 	}
-	if (defined $score_index) {
+	if ($score_index) {
 		push @args, 'score', $row->value($score_index);
 	}
-	if (defined $name_index) {
-		my $name = $unique ? 
-			generate_unique_name($row->value($name_index), $unique_name_counter) : 
-			$row->value($name_index);
+	if ($name_index) {
+		my $name =
+			$unique
+			? generate_unique_name( $row->value($name_index), $unique_name_counter )
+			: $row->value($name_index);
 		push @args, 'name', $name;
-	} elsif (defined $name_base) {
-		push @args, 'name', sprintf("%s_%07d", $name_base, $count);
 	}
-	if (defined $id_index) {
+	elsif ($name_base) {
+		push @args, 'name', sprintf( "%s_%07d", $name_base, $count );
+	}
+	if ($id_index) {
 		my $id = $row->value($id_index);
-		push @args, 'id', generate_unique_name($id, $unique_id_counter);
+		push @args, 'id', generate_unique_name( $id, $unique_id_counter );
 	}
-	if (defined $type_index) {
+	if ($type_index) {
 		push @args, 'type', $row->value($type_index);
-	} elsif (defined $type_base) {
+	}
+	elsif ($type_base) {
 		push @args, 'type', $type_base;
 	}
-	if (defined $source_index) {
+	if ($source_index) {
 		push @args, 'source', $row->value($source_index);
-	} elsif (defined $source_base) {
+	}
+	elsif ($source_base) {
 		push @args, 'source', $source_base;
 	}
 	if (@tag_indices) {
 		push @args, 'attributes', \@tag_indices;
 	}
-	
-		
+
 	# add to output
 	my $string = $row->gff_string(@args);
 	$Output->add_row($string) if length($string);
-		# weirdly, this should work, as the add_row will split the columns of 
-		# the gff string automatically
+
+	# weirdly, this should work, as the add_row will split the columns of
+	# the gff string automatically
 	$count++;
 }
 
-
-
-
 ### Finish
 $Input->close_fh;
 if ($sort_data) {
@@ -385,24 +388,23 @@ if ($sort_data) {
 	$Output->gsort_data;
 }
 unless ($outfile) {
-	$outfile = $Input->path . $Input->basename;
+	$outfile = sprintf "%s%s.gff", $Input->path, $Input->basename;
 }
 $outfile = $Output->write_file(
-	filename => $outfile, 
+	filename => $outfile,
 	gz       => $gz,
 );
 
-
-printf " Converted %s lines of input data to GFF file '%s'\n", 
+printf " Converted %s lines of input data to GFF file '%s'\n",
 	format_with_commas($count), $outfile;
 
-
 sub generate_unique_name {
-	my ($name, $counter) = @_;
+	my ( $name, $counter ) = @_;
 	my $new_name;
-			
+
 	# check uniqueness
-	if (exists $counter->{$name} ) {
+	if ( exists $counter->{$name} ) {
+
 		# we've encountered this name before
 		# generate a unique name by appending the count number
 		$counter->{$name} += 1;
@@ -417,7 +419,6 @@ sub generate_unique_name {
 	return $new_name;
 }
 
-
 __END__
 
 =head1 NAME
