@@ -24,7 +24,7 @@ eval {
 	$parallel = 1;
 };
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 print "\n This script will collect binned values across features\n\n";
 
@@ -151,6 +151,12 @@ if ($infile) {
 	{
 		$Data->feature($feature);
 	}
+
+	# set headers to true if file was explicitly not parsed
+	# since warnings will occur when we add columns and write to a file
+	if ( $parse == 0 ) {
+		$Data->headers(1);
+	}
 }
 else {
 	# generate a new file
@@ -168,8 +174,16 @@ my $startcolumn;    # this is now calculated separately for each datasets
 
 # Check output file name
 unless ($outfile) {
-	if ( $Data->basename ) {
-		$outfile = $Data->path . $Data->basename;
+	if ( $Data->filename ) {
+
+		# reuse the input filename and overwrite
+		$outfile = $Data->filename;
+	}
+	elsif ( $Data->basename ) {
+
+		# no filename but a basename is indicative of a parsed file
+		# so stick a txt extension on it, and write in current directory
+		$outfile = sprintf "%s.txt", $Data->basename;
 	}
 	else {
 		print STDERR " FATAL: No output file provided!\n";
